@@ -22,8 +22,9 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-#include <config.h>
+#include "config.h"
 
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -40,6 +41,8 @@
 
 #include <wayland-client.h>
 #include "shared/os-compatibility.h"
+#include "shared/xalloc.h"
+#include "shared/zalloc.h"
 
 struct device {
 	enum { KEYBOARD, POINTER } type;
@@ -82,20 +85,6 @@ static const struct wl_buffer_listener buffer_listener = {
 	buffer_release
 };
 
-static inline void *
-xzalloc(size_t s)
-{
-	void *p;
-
-	p = calloc(1, s);
-	if (p == NULL) {
-		fprintf(stderr, "%s: out of memory\n", program_invocation_short_name);
-		exit(EXIT_FAILURE);
-	}
-
-	return p;
-}
-
 static int
 attach_buffer(struct window *window, int width, int height)
 {
@@ -446,12 +435,13 @@ create_device(struct display *display, const char *time_desc, int type)
 
 	errno = 0;
 	start_time = strtoul(time_desc, &tail, 10);
-	if (errno)
+	if (errno || tail == time_desc)
 		goto error;
 
 	if (*tail == ':') {
-		end_time = strtoul(tail + 1, &tail, 10);
-		if (errno || *tail != '\0')
+		time_desc = tail + 1;
+		end_time = strtoul(time_desc, &tail, 10);
+		if (errno || tail == time_desc || *tail != '\0')
 			goto error;
 	} else if (*tail != '\0') {
 		goto error;
