@@ -50,15 +50,8 @@
 #include "shared/xalloc.h"
 #include "window.h"
 
-#ifndef EGL_WL_create_wayland_buffer_from_image
-#define EGL_WL_create_wayland_buffer_from_image 1
+#include "weston-egl-ext.h"
 
-#ifdef EGL_EGLEXT_PROTOTYPES
-EGLAPI struct wl_buffer * EGLAPIENTRY eglCreateWaylandBufferFromImageWL(EGLDisplay dpy, EGLImageKHR image);
-#endif
-typedef struct wl_buffer * (EGLAPIENTRYP PFNEGLCREATEWAYLANDBUFFERFROMIMAGEWL) (EGLDisplay dpy, EGLImageKHR image);
-
-#endif
 
 static int option_blit;
 
@@ -235,8 +228,7 @@ nested_buffer_reference(struct nested_buffer_reference *ref,
 		ref->buffer->busy_count--;
 		if (ref->buffer->busy_count == 0) {
 			assert(wl_resource_get_client(ref->buffer->resource));
-			wl_resource_queue_event(ref->buffer->resource,
-						WL_BUFFER_RELEASE);
+			wl_buffer_send_release(ref->buffer->resource);
 		}
 		wl_list_remove(&ref->destroy_listener.link);
 	}
@@ -755,7 +747,7 @@ nested_init_compositor(struct nested *nested)
 
 	nested->egl_display = display_get_egl_display(nested->display);
 	extensions = eglQueryString(nested->egl_display, EGL_EXTENSIONS);
-	if (strstr(extensions, "EGL_WL_bind_wayland_display") == NULL) {
+	if (!weston_check_egl_extension(extensions, "EGL_WL_bind_wayland_display")) {
 		fprintf(stderr, "no EGL_WL_bind_wayland_display extension\n");
 		return -1;
 	}
@@ -778,7 +770,7 @@ nested_init_compositor(struct nested *nested)
 		const char *func = "eglCreateWaylandBufferFromImageWL";
 		const char *ext = "EGL_WL_create_wayland_buffer_from_image";
 
-		if (strstr(extensions, ext)) {
+		if (weston_check_egl_extension(extensions, ext)) {
 			create_wayland_buffer_from_image =
 				(void *) eglGetProcAddress(func);
 			use_ss_renderer = 1;
