@@ -452,6 +452,7 @@ text_input_manager_notifier_destroy(struct wl_listener *listener, void *data)
 			     struct text_input_manager,
 			     destroy_listener);
 
+	wl_list_remove(&text_input_manager->destroy_listener.link);
 	wl_global_destroy(text_input_manager->text_input_manager_global);
 
 	free(text_input_manager);
@@ -679,9 +680,7 @@ input_method_context_grab_keyboard(struct wl_client *client,
 
 	context->keyboard = cr;
 
-	wl_keyboard_send_keymap(cr, WL_KEYBOARD_KEYMAP_FORMAT_XKB_V1,
-				keyboard->xkb_info->keymap_fd,
-				keyboard->xkb_info->keymap_size);
+	weston_keyboard_send_keymap(keyboard, cr);
 
 	if (keyboard->grab != &keyboard->default_grab) {
 		weston_keyboard_end_grab(keyboard);
@@ -1047,14 +1046,10 @@ text_backend_configuration(struct text_backend *text_backend)
 	struct weston_config *config = wet_get_config(text_backend->compositor);
 	struct weston_config_section *section;
 	char *client;
-	int ret;
 
 	section = weston_config_get_section(config,
 					    "input-method", NULL, NULL);
-	ret = asprintf(&client, "%s/weston-keyboard",
-		       weston_config_get_libexec_dir());
-	if (ret < 0)
-		client = NULL;
+	client = wet_get_binary_path("weston-keyboard");
 	weston_config_section_get_string(section, "path",
 					 &text_backend->input_method.path,
 					 client);
@@ -1064,6 +1059,8 @@ text_backend_configuration(struct text_backend *text_backend)
 WL_EXPORT void
 text_backend_destroy(struct text_backend *text_backend)
 {
+	wl_list_remove(&text_backend->seat_created_listener.link);
+
 	if (text_backend->input_method.client) {
 		/* disable respawn */
 		wl_list_remove(&text_backend->client_listener.link);
