@@ -59,6 +59,7 @@ extern "C" {
 
 #include "stdbool.h"
 #include "compositor.h"
+#include "plugin-registry.h"
 
 #define IVI_SUCCEEDED (0)
 #define IVI_FAILED (-1)
@@ -101,7 +102,7 @@ struct ivi_layout_layer_properties
 	int32_t dest_width;
 	int32_t dest_height;
 	enum wl_output_transform orientation;
-	uint32_t visibility;
+	bool visibility;
 	int32_t transition_type;
 	uint32_t transition_duration;
 	double start_alpha;
@@ -140,6 +141,8 @@ enum ivi_layout_transition_type{
 	IVI_LAYOUT_TRANSITION_MAX,
 };
 
+#define IVI_LAYOUT_API_NAME "ivi_layout_api_v1"
+
 struct ivi_layout_interface {
 
 	/**
@@ -572,6 +575,16 @@ struct ivi_layout_interface {
 				       struct ivi_layout_layer *removelayer);
 };
 
+static inline const struct ivi_layout_interface *
+ivi_layout_get_api(struct weston_compositor *compositor)
+{
+	const void *api;
+	api = weston_plugin_api_get(compositor, IVI_LAYOUT_API_NAME,
+				    sizeof(struct ivi_layout_interface));
+
+	return (const struct ivi_layout_interface *)api;
+}
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
