@@ -27,12 +27,14 @@
 #define WESTON_IVI_SHELL_H
 
 #include <stdbool.h>
+#include <stdint.h>
 
 #include "compositor.h"
 
 struct ivi_shell
 {
 	struct wl_listener destroy_listener;
+	struct wl_listener wake_listener;
 
 	struct weston_compositor *compositor;
 
@@ -73,4 +75,9 @@ void
 shell_surface_send_configure(struct weston_surface *surface,
 			     int32_t width, int32_t height);
 
+struct ivi_layout_surface;
+
+struct ivi_layout_surface *
+shell_get_ivi_layout_surface(struct weston_surface *surface);
+
 #endif /* WESTON_IVI_SHELL_H */
