@@ -58,6 +58,34 @@ get_udev_seat(struct udev_input *input, struct libinput_device *device)
 	return udev_seat_get_named(input, seat_name);
 }
 
+static struct weston_output *
+output_find_by_head_name(struct weston_compositor *compositor,
+			 const char *head_name)
+{
+	struct weston_output *output;
+	struct weston_head *head;
+
+	if (!head_name)
+		return NULL;
+
+	/* Only enabled outputs with connected heads.
+	 * This means force-enabled outputs but with disconnected heads
+	 * will be ignored; if the touchscreen doesn't have a video signal,
+	 * touching it is meaningless.
+	 */
+	wl_list_for_each(output, &compositor->output_list, link) {
+		wl_list_for_each(head, &output->head_list, output_link) {
+			if (!weston_head_is_connected(head))
+				continue;
+
+			if (strcmp(head_name, head->name) == 0)
+				return output;
+		}
+	}
+
+	return NULL;
+}
+
 static void
 device_added(struct udev_input *input, struct libinput_device *libinput_device)
 {
@@ -95,11 +123,10 @@ device_added(struct udev_input *input, struct libinput_device *libinput_device)
 	output_name = libinput_device_get_output_name(libinput_device);
 	if (output_name) {
 		device->output_name = strdup(output_name);
-		wl_list_for_each(output, &c->output_list, link)
-			if (output->name &&
-			    strcmp(output->name, device->output_name) == 0)
-				evdev_device_set_output(device, output);
-	} else if (device->output == NULL && !wl_list_empty(&c->output_list)) {
+		output = output_find_by_head_name(c, output_name);
+		evdev_device_set_output(device, output);
+	} else if (!wl_list_empty(&c->output_list)) {
+		/* default assignment to an arbitrary output */
 		output = container_of(c->output_list.next,
 				      struct weston_output, link);
 		evdev_device_set_output(device, output);
@@ -356,23 +383,51 @@ udev_seat_led_update(struct weston_seat *seat_base, enum weston_led leds)
 		evdev_led_update(device, leds);
 }
 
+static void
+udev_seat_output_changed(struct udev_seat *seat, struct weston_output *output)
+{
+	struct evdev_device *device;
+	struct weston_output *found;
+
+	wl_list_for_each(device, &seat->devices_list, link) {
+		/* If we find any input device without an associated output
+		 * or an output name to associate with, just tie it with the
+		 * output we got here - the default assingment.
+		 */
+		if (!device->output_name) {
+			if (!device->output)
+				evdev_device_set_output(device, output);
+
+			continue;
+		}
+
+		/* Update all devices' output associations, may they gain or
+		 * lose it.
+		 */
+		found = output_find_by_head_name(output->compositor,
+						 device->output_name);
+		evdev_device_set_output(device, found);
+	}
+}
+
 static void
 notify_output_create(struct wl_listener *listener, void *data)
 {
 	struct udev_seat *seat = container_of(listener, struct udev_seat,
 					      output_create_listener);
-	struct evdev_device *device;
 	struct weston_output *output = data;
 
-	wl_list_for_each(device, &seat->devices_list, link) {
-		if (device->output_name &&
-		    strcmp(output->name, device->output_name) == 0) {
-			evdev_device_set_output(device, output);
-		}
+	udev_seat_output_changed(seat, output);
+}
 
-		if (device->output_name == NULL && device->output == NULL)
-			evdev_device_set_output(device, output);
-	}
+static void
+notify_output_heads_changed(struct wl_listener *listener, void *data)
+{
+	struct udev_seat *seat = container_of(listener, struct udev_seat,
+					      output_heads_listener);
+	struct weston_output *output = data;
+
+	udev_seat_output_changed(seat, output);
 }
 
 static struct udev_seat *
@@ -392,6 +447,10 @@ udev_seat_create(struct udev_input *input, const char *seat_name)
 	wl_signal_add(&c->output_created_signal,
 		      &seat->output_create_listener);
 
+	seat->output_heads_listener.notify = notify_output_heads_changed;
+	wl_signal_add(&c->output_heads_changed_signal,
+		      &seat->output_heads_listener);
+
 	wl_list_init(&seat->devices_list);
 
 	return seat;
@@ -409,6 +468,7 @@ udev_seat_destroy(struct udev_seat *seat)
 	udev_seat_remove_devices(seat);
 	weston_seat_release(&seat->base);
 	wl_list_remove(&seat->output_create_listener.link);
+	wl_list_remove(&seat->output_heads_listener.link);
 	free(seat);
 }
 
