@@ -32,10 +32,15 @@
 #ifndef DRM_FORMAT_MOD_INVALID
 #define DRM_FORMAT_MOD_INVALID ((1ULL<<56) - 1)
 #endif
+#ifndef DRM_FORMAT_MOD_LINEAR
+#define DRM_FORMAT_MOD_LINEAR 0
+#endif
 
 struct linux_dmabuf_buffer;
 typedef void (*dmabuf_user_data_destroy_func)(
 			struct linux_dmabuf_buffer *buffer);
+typedef void (*dmabuf_gem_handle_close_func)(
+			struct linux_dmabuf_buffer *buffer);
 
 struct dmabuf_attributes {
 	int32_t width;
@@ -47,6 +52,7 @@ struct dmabuf_attributes {
 	uint32_t offset[MAX_DMABUF_PLANES];
 	uint32_t stride[MAX_DMABUF_PLANES];
 	uint64_t modifier[MAX_DMABUF_PLANES];
+	uint64_t dtrc_meta;
 };
 
 struct linux_dmabuf_buffer {
@@ -72,6 +78,8 @@ struct linux_dmabuf_buffer {
 	 * feasible to scan it out directly. This would improve the
 	 * possibilities to successfully scan out, avoiding compositing.
 	 */
+	uint32_t gem_handles[MAX_DMABUF_PLANES];
+	dmabuf_gem_handle_close_func gem_handle_close_func;
 };
 
 int
@@ -80,6 +88,10 @@ linux_dmabuf_setup(struct weston_compositor *compositor);
 struct linux_dmabuf_buffer *
 linux_dmabuf_buffer_get(struct wl_resource *resource);
 
+void
+linux_dmabuf_buffer_gem_handle_close_cb(struct linux_dmabuf_buffer *buffer,
+				  dmabuf_gem_handle_close_func func);
+
 void
 linux_dmabuf_buffer_set_user_data(struct linux_dmabuf_buffer *buffer,
 				  void *data,
