@@ -26,10 +26,11 @@
 #include "config.h"
 
 #include <string.h>
+#include <stdint.h>
 #include <stdio.h>
 
 #include "weston-test-client-helper.h"
-#include "text-client-protocol.h"
+#include "text-input-unstable-v1-client-protocol.h"
 
 struct text_input_state {
 	int activated;
@@ -38,7 +39,7 @@ struct text_input_state {
 
 static void
 text_input_commit_string(void *data,
-			 struct wl_text_input *text_input,
+			 struct zwp_text_input_v1 *text_input,
 			 uint32_t serial,
 			 const char *text)
 {
@@ -46,7 +47,7 @@ text_input_commit_string(void *data,
 
 static void
 text_input_preedit_string(void *data,
-			  struct wl_text_input *text_input,
+			  struct zwp_text_input_v1 *text_input,
 			  uint32_t serial,
 			  const char *text,
 			  const char *commit)
@@ -55,7 +56,7 @@ text_input_preedit_string(void *data,
 
 static void
 text_input_delete_surrounding_text(void *data,
-				   struct wl_text_input *text_input,
+				   struct zwp_text_input_v1 *text_input,
 				   int32_t index,
 				   uint32_t length)
 {
@@ -63,7 +64,7 @@ text_input_delete_surrounding_text(void *data,
 
 static void
 text_input_cursor_position(void *data,
-			   struct wl_text_input *text_input,
+			   struct zwp_text_input_v1 *text_input,
 			   int32_t index,
 			   int32_t anchor)
 {
@@ -71,7 +72,7 @@ text_input_cursor_position(void *data,
 
 static void
 text_input_preedit_styling(void *data,
-			   struct wl_text_input *text_input,
+			   struct zwp_text_input_v1 *text_input,
 			   uint32_t index,
 			   uint32_t length,
 			   uint32_t style)
@@ -80,21 +81,21 @@ text_input_preedit_styling(void *data,
 
 static void
 text_input_preedit_cursor(void *data,
-			  struct wl_text_input *text_input,
+			  struct zwp_text_input_v1 *text_input,
 			  int32_t index)
 {
 }
 
 static void
 text_input_modifiers_map(void *data,
-			 struct wl_text_input *text_input,
+			 struct zwp_text_input_v1 *text_input,
 			 struct wl_array *map)
 {
 }
 
 static void
 text_input_keysym(void *data,
-		  struct wl_text_input *text_input,
+		  struct zwp_text_input_v1 *text_input,
 		  uint32_t serial,
 		  uint32_t time,
 		  uint32_t sym,
@@ -105,7 +106,7 @@ text_input_keysym(void *data,
 
 static void
 text_input_enter(void *data,
-		 struct wl_text_input *text_input,
+		 struct zwp_text_input_v1 *text_input,
 		 struct wl_surface *surface)
 
 {
@@ -118,7 +119,7 @@ text_input_enter(void *data,
 
 static void
 text_input_leave(void *data,
-		 struct wl_text_input *text_input)
+		 struct zwp_text_input_v1 *text_input)
 {
 	struct text_input_state *state = data;
 
@@ -127,14 +128,14 @@ text_input_leave(void *data,
 
 static void
 text_input_input_panel_state(void *data,
-			     struct wl_text_input *text_input,
+			     struct zwp_text_input_v1 *text_input,
 			     uint32_t state)
 {
 }
 
 static void
 text_input_language(void *data,
-		    struct wl_text_input *text_input,
+		    struct zwp_text_input_v1 *text_input,
 		    uint32_t serial,
 		    const char *language)
 {
@@ -142,13 +143,13 @@ text_input_language(void *data,
 
 static void
 text_input_text_direction(void *data,
-			  struct wl_text_input *text_input,
+			  struct zwp_text_input_v1 *text_input,
 			  uint32_t serial,
 			  uint32_t direction)
 {
 }
 
-static const struct wl_text_input_listener text_input_listener = {
+static const struct zwp_text_input_v1_listener text_input_listener = {
 	text_input_enter,
 	text_input_leave,
 	text_input_modifiers_map,
@@ -168,8 +169,8 @@ TEST(text_test)
 {
 	struct client *client;
 	struct global *global;
-	struct wl_text_input_manager *factory;
-	struct wl_text_input *text_input;
+	struct zwp_text_input_manager_v1 *factory;
+	struct zwp_text_input_v1 *text_input;
 	struct text_input_state state;
 
 	client = create_client_and_test_surface(100, 100, 100, 100);
@@ -177,17 +178,19 @@ TEST(text_test)
 
 	factory = NULL;
 	wl_list_for_each(global, &client->global_list, link) {
-		if (strcmp(global->interface, "wl_text_input_manager") == 0)
+		if (strcmp(global->interface, "zwp_text_input_manager_v1") == 0)
 			factory = wl_registry_bind(client->wl_registry,
 						   global->name,
-						   &wl_text_input_manager_interface, 1);
+						   &zwp_text_input_manager_v1_interface, 1);
 	}
 
 	assert(factory);
 
 	memset(&state, 0, sizeof state);
-	text_input = wl_text_input_manager_create_text_input(factory);
-	wl_text_input_add_listener(text_input, &text_input_listener, &state);
+	text_input = zwp_text_input_manager_v1_create_text_input(factory);
+	zwp_text_input_v1_add_listener(text_input,
+				       &text_input_listener,
+				       &state);
 
 	/* Make sure our test surface has keyboard focus. */
 	weston_test_activate_surface(client->test->weston_test,
@@ -196,19 +199,19 @@ TEST(text_test)
 	assert(client->input->keyboard->focus == client->surface);
 
 	/* Activate test model and make sure we get enter event. */
-	wl_text_input_activate(text_input, client->input->wl_seat,
-			       client->surface->wl_surface);
+	zwp_text_input_v1_activate(text_input, client->input->wl_seat,
+				   client->surface->wl_surface);
 	client_roundtrip(client);
 	assert(state.activated == 1 && state.deactivated == 0);
 
 	/* Deactivate test model and make sure we get leave event. */
-	wl_text_input_deactivate(text_input, client->input->wl_seat);
+	zwp_text_input_v1_deactivate(text_input, client->input->wl_seat);
 	client_roundtrip(client);
 	assert(state.activated == 1 && state.deactivated == 1);
 
 	/* Activate test model again. */
-	wl_text_input_activate(text_input, client->input->wl_seat,
-			       client->surface->wl_surface);
+	zwp_text_input_v1_activate(text_input, client->input->wl_seat,
+				   client->surface->wl_surface);
 	client_roundtrip(client);
 	assert(state.activated == 2 && state.deactivated == 1);
 
