@@ -148,12 +148,15 @@ pointer_handle_leave(void *data, struct wl_pointer *wl_pointer,
 
 static void
 pointer_handle_motion(void *data, struct wl_pointer *wl_pointer,
-		      uint32_t time, wl_fixed_t x, wl_fixed_t y)
+		      uint32_t time_msec, wl_fixed_t x, wl_fixed_t y)
 {
 	struct pointer *pointer = data;
 
 	pointer->x = wl_fixed_to_int(x);
 	pointer->y = wl_fixed_to_int(y);
+	pointer->motion_time_msec = time_msec;
+	pointer->motion_time_timespec = pointer->input_timestamp;
+	pointer->input_timestamp = (struct timespec) { 0 };
 
 	fprintf(stderr, "test-client: got pointer motion %d %d\n",
 		pointer->x, pointer->y);
@@ -161,13 +164,16 @@ pointer_handle_motion(void *data, struct wl_pointer *wl_pointer,
 
 static void
 pointer_handle_button(void *data, struct wl_pointer *wl_pointer,
-		      uint32_t serial, uint32_t time, uint32_t button,
+		      uint32_t serial, uint32_t time_msec, uint32_t button,
 		      uint32_t state)
 {
 	struct pointer *pointer = data;
 
 	pointer->button = button;
 	pointer->state = state;
+	pointer->button_time_msec = time_msec;
+	pointer->button_time_timespec = pointer->input_timestamp;
+	pointer->input_timestamp = (struct timespec) { 0 };
 
 	fprintf(stderr, "test-client: got pointer button %u %u\n",
 		button, state);
@@ -175,8 +181,16 @@ pointer_handle_button(void *data, struct wl_pointer *wl_pointer,
 
 static void
 pointer_handle_axis(void *data, struct wl_pointer *wl_pointer,
-		    uint32_t time, uint32_t axis, wl_fixed_t value)
+		    uint32_t time_msec, uint32_t axis, wl_fixed_t value)
 {
+	struct pointer *pointer = data;
+
+	pointer->axis = axis;
+	pointer->axis_value = wl_fixed_to_double(value);
+	pointer->axis_time_msec = time_msec;
+	pointer->axis_time_timespec = pointer->input_timestamp;
+	pointer->input_timestamp = (struct timespec) { 0 };
+
 	fprintf(stderr, "test-client: got pointer axis %u %f\n",
 		axis, wl_fixed_to_double(value));
 }
@@ -196,9 +210,16 @@ pointer_handle_axis_source(void *data, struct wl_pointer *wl_pointer,
 
 static void
 pointer_handle_axis_stop(void *data, struct wl_pointer *wl_pointer,
-			 uint32_t time, uint32_t axis)
+			 uint32_t time_msec, uint32_t axis)
 {
-	fprintf(stderr, "test-client: got pointer axis stop\n");
+	struct pointer *pointer = data;
+
+	pointer->axis = axis;
+	pointer->axis_stop_time_msec = time_msec;
+	pointer->axis_stop_time_timespec = pointer->input_timestamp;
+	pointer->input_timestamp = (struct timespec) { 0 };
+
+	fprintf(stderr, "test-client: got pointer axis stop %u\n", axis);
 }
 
 static void
@@ -260,13 +281,16 @@ keyboard_handle_leave(void *data, struct wl_keyboard *wl_keyboard,
 
 static void
 keyboard_handle_key(void *data, struct wl_keyboard *wl_keyboard,
-		    uint32_t serial, uint32_t time, uint32_t key,
+		    uint32_t serial, uint32_t time_msec, uint32_t key,
 		    uint32_t state)
 {
 	struct keyboard *keyboard = data;
 
 	keyboard->key = key;
 	keyboard->state = state;
+	keyboard->key_time_msec = time_msec;
+	keyboard->key_time_timespec = keyboard->input_timestamp;
+	keyboard->input_timestamp = (struct timespec) { 0 };
 
 	fprintf(stderr, "test-client: got keyboard key %u %u\n", key, state);
 }
@@ -312,14 +336,18 @@ static const struct wl_keyboard_listener keyboard_listener = {
 
 static void
 touch_handle_down(void *data, struct wl_touch *wl_touch,
-		  uint32_t serial, uint32_t time, struct wl_surface *surface,
-		  int32_t id, wl_fixed_t x_w, wl_fixed_t y_w)
+		  uint32_t serial, uint32_t time_msec,
+		  struct wl_surface *surface, int32_t id,
+		  wl_fixed_t x_w, wl_fixed_t y_w)
 {
 	struct touch *touch = data;
 
 	touch->down_x = wl_fixed_to_int(x_w);
 	touch->down_y = wl_fixed_to_int(y_w);
 	touch->id = id;
+	touch->down_time_msec = time_msec;
+	touch->down_time_timespec = touch->input_timestamp;
+	touch->input_timestamp = (struct timespec) { 0 };
 
 	fprintf(stderr, "test-client: got touch down %d %d, surf: %p, id: %d\n",
 		touch->down_x, touch->down_y, surface, id);
@@ -327,21 +355,28 @@ touch_handle_down(void *data, struct wl_touch *wl_touch,
 
 static void
 touch_handle_up(void *data, struct wl_touch *wl_touch,
-		uint32_t serial, uint32_t time, int32_t id)
+		uint32_t serial, uint32_t time_msec, int32_t id)
 {
 	struct touch *touch = data;
 	touch->up_id = id;
+	touch->up_time_msec = time_msec;
+	touch->up_time_timespec = touch->input_timestamp;
+	touch->input_timestamp = (struct timespec) { 0 };
 
 	fprintf(stderr, "test-client: got touch up, id: %d\n", id);
 }
 
 static void
 touch_handle_motion(void *data, struct wl_touch *wl_touch,
-		    uint32_t time, int32_t id, wl_fixed_t x_w, wl_fixed_t y_w)
+		    uint32_t time_msec, int32_t id,
+		    wl_fixed_t x_w, wl_fixed_t y_w)
 {
 	struct touch *touch = data;
 	touch->x = wl_fixed_to_int(x_w);
 	touch->y = wl_fixed_to_int(y_w);
+	touch->motion_time_msec = time_msec;
+	touch->motion_time_timespec = touch->input_timestamp;
+	touch->input_timestamp = (struct timespec) { 0 };
 
 	fprintf(stderr, "test-client: got touch motion, %d %d, id: %d\n",
 		touch->x, touch->y, id);
@@ -519,6 +554,27 @@ static const struct weston_test_listener test_listener = {
 	test_handle_capture_screenshot_done,
 };
 
+static void
+input_destroy(struct input *inp)
+{
+	if (inp->pointer) {
+		wl_pointer_release(inp->pointer->wl_pointer);
+		free(inp->pointer);
+	}
+	if (inp->keyboard) {
+		wl_keyboard_release(inp->keyboard->wl_keyboard);
+		free(inp->keyboard);
+	}
+	if (inp->touch) {
+		wl_touch_release(inp->touch->wl_touch);
+		free(inp->touch);
+	}
+	wl_list_remove(&inp->link);
+	wl_seat_release(inp->wl_seat);
+	free(inp->seat_name);
+	free(inp);
+}
+
 static void
 input_update_devices(struct input *input)
 {
@@ -595,6 +651,15 @@ seat_handle_name(void *data, struct wl_seat *seat, const char *name)
 	input->seat_name = strdup(name);
 	assert(input->seat_name && "No memory");
 
+	/* We only update the devices and set client input for the test seat */
+	if (strcmp(name, "test-seat") == 0) {
+		assert(!input->client->input &&
+		       "Multiple test seats detected!");
+
+		input_update_devices(input);
+		input->client->input = input;
+	}
+
 	fprintf(stderr, "test-client: got seat %p name: \'%s\'\n",
 		input, name);
 }
@@ -686,6 +751,8 @@ handle_global(void *data, struct wl_registry *registry,
 					 &wl_compositor_interface, version);
 	} else if (strcmp(interface, "wl_seat") == 0) {
 		input = xzalloc(sizeof *input);
+		input->client = client;
+		input->global_name = global->name;
 		input->wl_seat =
 			wl_registry_bind(registry, id,
 					 &wl_seat_interface, version);
@@ -716,8 +783,59 @@ handle_global(void *data, struct wl_registry *registry,
 	}
 }
 
+static struct global *
+client_find_global_with_name(struct client *client, uint32_t name)
+{
+	struct global *global;
+
+	wl_list_for_each(global, &client->global_list, link) {
+		if (global->name == name)
+			return global;
+	}
+
+	return NULL;
+}
+
+static struct input *
+client_find_input_with_name(struct client *client, uint32_t name)
+{
+	struct input *input;
+
+	wl_list_for_each(input, &client->inputs, link) {
+		if (input->global_name == name)
+			return input;
+	}
+
+	return NULL;
+}
+
+static void
+handle_global_remove(void *data, struct wl_registry *registry, uint32_t name)
+{
+	struct client *client = data;
+	struct global *global;
+	struct input *input;
+
+	global = client_find_global_with_name(client, name);
+	assert(global && "Request to remove unknown global");
+
+	if (strcmp(global->interface, "wl_seat") == 0) {
+		input = client_find_input_with_name(client, name);
+		if (input) {
+			if (client->input == input)
+				client->input = NULL;
+			input_destroy(input);
+		}
+	}
+
+	wl_list_remove(&global->link);
+	free(global->interface);
+	free(global);
+}
+
 static const struct wl_registry_listener registry_listener = {
-	handle_global
+	handle_global,
+	handle_global_remove,
 };
 
 void
@@ -790,34 +908,6 @@ log_handler(const char *fmt, va_list args)
 	vfprintf(stderr, fmt, args);
 }
 
-static void
-input_destroy(struct input *inp)
-{
-	wl_list_remove(&inp->link);
-	wl_seat_destroy(inp->wl_seat);
-	free(inp);
-}
-
-/* find the test-seat and set it in client.
- * Destroy other inputs */
-static void
-client_set_input(struct client *cl)
-{
-	struct input *inp, *inptmp;
-	wl_list_for_each_safe(inp, inptmp, &cl->inputs, link) {
-		assert(inp->seat_name && "BUG: input with no name");
-		if (strcmp(inp->seat_name, "test-seat") == 0) {
-			cl->input = inp;
-			input_update_devices(inp);
-		} else {
-			input_destroy(inp);
-		}
-	}
-
-	/* we keep only one input */
-	assert(wl_list_length(&cl->inputs) == 1);
-}
-
 struct client *
 create_client(void)
 {
@@ -843,9 +933,6 @@ create_client(void)
 	 * events */
 	client_roundtrip(client);
 
-	/* find the right input for us */
-	client_set_input(client);
-
 	/* must have WL_SHM_FORMAT_ARGB32 */
 	assert(client->has_argb);
 
@@ -864,6 +951,25 @@ create_client(void)
 	return client;
 }
 
+struct surface *
+create_test_surface(struct client *client)
+{
+	struct surface *surface;
+
+	surface = xzalloc(sizeof *surface);
+
+	surface->wl_surface =
+		wl_compositor_create_surface(client->wl_compositor);
+	assert(surface->wl_surface);
+
+	wl_surface_add_listener(surface->wl_surface, &surface_listener,
+				surface);
+
+	wl_surface_set_user_data(surface->wl_surface, surface);
+
+	return surface;
+}
+
 struct client *
 create_client_and_test_surface(int x, int y, int width, int height)
 {
@@ -875,16 +981,8 @@ create_client_and_test_surface(int x, int y, int width, int height)
 	client = create_client();
 
 	/* initialize the client surface */
-	surface = xzalloc(sizeof *surface);
-	surface->wl_surface =
-		wl_compositor_create_surface(client->wl_compositor);
-	assert(surface->wl_surface);
-
-	wl_surface_add_listener(surface->wl_surface, &surface_listener,
-				surface);
-
+	surface = create_test_surface(client);
 	client->surface = surface;
-	wl_surface_set_user_data(surface->wl_surface, surface);
 
 	surface->width = width;
 	surface->height = height;
